﻿using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using System.Text;
using IdeaBlade.EntityModel;

// This sample uses Employee and Territory from the NorthwindIB database and shows how to work with
// many-to-many mapping without payload.
//
// In a many-to-many mapping without payload the mapping table only contains foreign keys to the other tables and no other
// columns. The mapping table is hidden from the application and no entity is created. This sample uses the EmployeeTerritoryNoPayload 
// table (hidden in the EDMX model). You can use Employee.Territory.Add() or Territory.Employee.Add() to create a new mapping. When 
// saved, a new row in the EmployeeTerritory table will automatically be inserted.
//
//   This sample does not save to the database, but you can do so easily by caling 
//   _entityManager.SaveChangesAsync();

namespace SilverlightConsole {
  public class CodeSamples {
    public CodeSamples() {
    }

    public void Start() {
      Samples();
    }

    private void Samples() {
      SLConsole.WriteLine("Note: The server may take a few seconds to initialize.\n");

      // The next three samples use the EmployeeTerritoryNoPayload mapping table
      EmployeeTerritoryNavigationNoPayload();
      AddTerritoryNoPayload();
      DeleteTerritoryNoPayload();
    }

    // The three sample methods below are where the "many-to-many" operations take place.
    private async void EmployeeTerritoryNavigationNoPayload() {
      SLConsole.WriteLine("EmployeeTerritoryNavigationNoPayload Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories so we can iterate over them below
      var employee = await _entityManager.Employees
        .Include("Territories")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      // employee.Territories is available immediately because it was cached by .Include("Territories")
      var territories = employee.Territories.ToList();

      SLConsole.WriteLine("Employee " + employee.FirstName + "  " + employee.LastName
        + " is assigned to " + employee.Territories.Count().ToString() + " territories:");

      foreach (var territory in territories) {
        SLConsole.WriteLine(territory.TerritoryDescription.Trim());
      }
      SLConsole.WriteLine();
    }

    private async void AddTerritoryNoPayload() {
      SLConsole.WriteLine("AddTerritoryNoPayLoad Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories
      var employee = await _entityManager.Employees
        .Include("Territories")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      SLConsole.WriteLine("Employee " + employee.FirstName + " " + employee.LastName
        + " is currently assigned to " + employee.Territories.Count() + " territories.");

      // Create a new Territory
      var territoryToBeAdded = new Territory();
      territoryToBeAdded.TerritoryDescription = "Oakland";
      territoryToBeAdded.RegionID = 2;
      // Remember to add it to the EntityManager
      _entityManager.AddEntity(territoryToBeAdded);

      SLConsole.WriteLine("Assigning Territory [" + territoryToBeAdded.TerritoryDescription.Trim()
        + "] from " + employee.FirstName + " " + employee.LastName);

      // This creates a new mapping between this Employee and Territory
      // A new row will be addded to the EmployeeTerritoryNoPayload table when saved
      employee.Territories.Add(territoryToBeAdded);

      SLConsole.WriteLine("Now " + employee.FirstName + " " + employee.LastName + " is currently assigned to "
        + employee.Territories.Count() + " territories.");
      SLConsole.WriteLine();
    }

    private async void DeleteTerritoryNoPayload() {
      SLConsole.WriteLine("DeleteTerritoryNoPayload Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories
      var employee = await _entityManager.Employees
        .Include("Territories")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      SLConsole.WriteLine("Employee " + employee.FirstName + " " + employee.LastName
        + " is currently assigned to " + employee.Territories.Count() + " territories.");

      Territory territoryToBeRemoved = employee.Territories.First();

      SLConsole.WriteLine("Unassigning Territory [" + territoryToBeRemoved.TerritoryDescription.Trim()
        + "] from " + employee.FirstName + " " + employee.LastName);

      // This removes the mapping between this Employee and Terrority
      // The row will be deleted from the EmployeeTerritoryNoPayload table when saved
      // Note that in this example, we did not delete any Territories, only the mapping between Employee and Territory
      employee.Territories.Remove(territoryToBeRemoved);

      SLConsole.WriteLine("Now " + employee.FirstName + " " + employee.LastName + " is currently assigned to "
        + employee.Territories.Count() + " territories.");
      SLConsole.WriteLine();
    }

    private NorthwindIBEntities _entityManager = new NorthwindIBEntities();
  }
}
