﻿Imports System.ComponentModel
Imports System.Text
Imports IdeaBlade.EntityModel

' This sample uses Employee and Territory from the NorthwindIB database and shows how to work with
' many-to-many mapping without payload.
'
' In a many-to-many mapping without payload the mapping table only contains foreign keys to the other tables and no other
' columns. The mapping table is hidden from the application and no entity is created. This sample uses the EmployeeTerritoryNoPayload 
' table (hidden in the EDMX model). You can use Employee.Territory.Add() or Territory.Employee.Add() to create a new mapping. When 
' saved, a new row in the EmployeeTerritory table will automatically be inserted.
'
'   This sample does not save to the database, but you can do so easily by caling 
'   _entityManager.SaveChangesAsync();

Namespace SilverlightConsole
  Public Class CodeSamples
	Public Sub New()
	End Sub

	Public Sub Start()
      Samples()
	End Sub

    Private Sub Samples()
      ' The next three samples use the EmployeeTerritoryNoPayload mapping table
      EmployeeTerritoryNavigationNoPayload()
      AddTerritoryNoPayload()
      DeleteTerritoryNoPayload()
    End Sub

    ' The three sample methods below are where the "many-to-many" operations take place.
    Private Async Sub EmployeeTerritoryNavigationNoPayload()
      SLConsole.WriteLine("EmployeeTerritoryNavigationNoPayload Started...")
      _entityManager.Clear()

      ' Get an Employee and include their Territories so we can iterate over them below
      Dim employee = Await _entityManager.Employees.Include("Territories").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

      ' employee.Territories is available immediately because it was cached by .Include("Territories")
      Dim territories = employee.Territories.ToList()

      SLConsole.WriteLine("Employee " & employee.FirstName + "  " + employee.LastName & " is assigned to " & employee.Territories.Count().ToString() & " territories:")

      For Each territory In territories
        SLConsole.WriteLine(territory.TerritoryDescription.Trim())
      Next territory
      SLConsole.WriteLine()
    End Sub

    Private Async Sub AddTerritoryNoPayload()
      SLConsole.WriteLine("AddTerritoryNoPayLoad Started...")
      _entityManager.Clear()

      ' Get an Employee and include their Territories
      Dim employee = Await _entityManager.Employees.Include("Territories").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

      SLConsole.WriteLine("Employee " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.Territories.Count() & " territories.")

      ' Create a new Territory
      Dim territoryToBeAdded = New Territory()
      territoryToBeAdded.TerritoryDescription = "Oakland"
      territoryToBeAdded.RegionID = 2
      ' Remember to add it to the EntityManager
      _entityManager.AddEntity(territoryToBeAdded)

      SLConsole.WriteLine("Assigning Territory [" & territoryToBeAdded.TerritoryDescription.Trim() & "] from " & employee.FirstName & " " & employee.LastName)

      ' This creates a new mapping between this Employee and Territory
      ' A new row will be addded to the EmployeeTerritoryNoPayload table when saved
      employee.Territories.Add(territoryToBeAdded)

      SLConsole.WriteLine("Now " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.Territories.Count() & " territories.")
      SLConsole.WriteLine()
    End Sub

    Private Async Sub DeleteTerritoryNoPayload()
      SLConsole.WriteLine("DeleteTerritoryNoPayload Started...")
      _entityManager.Clear()

      ' Get an Employee and include their Territories
      Dim employee = Await _entityManager.Employees.Include("Territories").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

      SLConsole.WriteLine("Employee " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.Territories.Count() & " territories.")

      Dim territoryToBeRemoved As Territory = employee.Territories.First()

      SLConsole.WriteLine("Unassigning Territory [" & territoryToBeRemoved.TerritoryDescription.Trim() & "] from " & employee.FirstName & " " & employee.LastName)

      ' This removes the mapping between this Employee and Terrority
      ' The row will be deleted from the EmployeeTerritoryNoPayload table when saved
      ' Note that in this example, we did not delete any Territories, only the mapping between Employee and Territory
      employee.Territories.Remove(territoryToBeRemoved)

      SLConsole.WriteLine("Now " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.Territories.Count() & " territories.")
      SLConsole.WriteLine()
    End Sub

    Private _entityManager As New NorthwindIBEntities()
  End Class
End Namespace
